--[[
	From cs50 2020
	Games Track
	Pong

	"The Low-Res Update" tutorial by Colten Ogden
	written by Josh Borthick
	jborthick019919@gmail.com

	Originally programmed in 1972 by Atari.  Features two
	paddles, controlled by players, with the goal of getting
	the ball past the opponent's paddle.  First to 10 points
	wins.

	This game built to more closely resemble an NES than the
	original Pong machine or the Atari 2600.  NES resolution
	on 16:9 screen to retro look but nice feel.
]]

-- Push allows drawing at any virtual resolution, rather
-- than whatever the window size is.
-- https://www.github.com/ulydev/push

-- External files to include.
Class = require "class"
push = require "push"

require "Ball"
require "Paddle"

-- Globals.
WINDOW_WIDTH = 1280
WINDOW_HEIGHT = 720

VIRTUAL_WIDTH = 432
VIRTUAL_HEIGHT = 243

PADDLE_SPEED = 200

function love.load()
	-- 'Nearest neighbor' filtering for crisper look.
	love.graphics.setDefaultFilter("nearest", "nearest")

	-- Fonts.
	smallFont = love.graphics.newFont("Retro Gaming.ttf", 8)
	scoreFont = love.graphics.newFont("Retro Gaming.ttf", 32)

	-- Initializes window to virtual resolution at actual window size.
    push:setupScreen(VIRTUAL_WIDTH, VIRTUAL_HEIGHT, WINDOW_WIDTH, WINDOW_HEIGHT, {
    	fullscreen = false,
    	vsync = true,	
    	resizable = false
    })

	math.randomseed(os.time())

	-- Player scores.
	player1Score = 0
	player2Score = 0

	-- Starting parameters.
	paddle1 = Paddle(5, 20, 5, 20)
	paddle2 = Paddle(VIRTUAL_WIDTH - 10, VIRTUAL_HEIGHT - 40, 5, 20)
	ball = Ball(VIRTUAL_WIDTH / 2 - 2, VIRTUAL_HEIGHT / 2 - 2, 5, 5)
	gameState = "start"
end

function love.update(dt)
	-- Player 1 movement.
	if love.keyboard.isDown("w") then
		paddle1.dy = -PADDLE_SPEED
	elseif love.keyboard.isDown("s") then
		paddle1.dy = PADDLE_SPEED
	else
		paddle1.dy = 0
	end

	-- Player 2 movement.
	if love.keyboard.isDown("up") then  
		paddle2.dy = -PADDLE_SPEED
	elseif love.keyboard.isDown("down") then 
		paddle2.dy = PADDLE_SPEED
	else
		paddle2.dy = 0
	end

	-- Ball and paddle movement.
	paddle1:update(dt)
	paddle2:update(dt)
	ball:update(dt)
end

function love.keypressed(key)
	-- Quit the game.
	if key == "escape" then 
		love.event.quit()
	-- Play or start a new game.
	elseif key == "enter" or key == "return" then
		if gameState == "start" then 
			gameState = "play"
		elseif gameState == "play" or gameState == "pause" then
			paddle1:reset1()
			paddle2:reset2()
			ball:reset()
			gameState = "start"
		end
	-- Pause the game.
	elseif key == "space" then 
		if gameState == "play" then 
			gameState = "pause"
		elseif gameState == "pause" then 
			gameState = "play" 
		end
	end
end

function love.draw()
	push:apply("start")

	-- BG color and game title.
	love.graphics.clear(0.16, 0.18, 0.20, 1.0)

	if gameState == "start" or gameState == "pause" then 
		love.graphics.setFont(smallFont)
		love.graphics.printf("Hello Pong!\nReturn to start\nSpace to pause", 0, 20, VIRTUAL_WIDTH, "center")

	    -- Score.
		love.graphics.setFont(scoreFont)
		love.graphics.print(player1Score, VIRTUAL_WIDTH / 2 - 59, VIRTUAL_HEIGHT / 3)
		love.graphics.print(player2Score, VIRTUAL_WIDTH / 2 + 30, VIRTUAL_HEIGHT / 3)
	end

	-- Ball, paddle 1, paddle 2.
	paddle1:render()
	paddle2:render()
	ball:render()

    push:apply("end")
end